#!/usr/bin/env python3
# -*- coding: utf-8; -*-




import katoptron.CMAME.model as model

import katoptron as m
import tbox
import tbox.gmsh as gmsh
import shlex
import subprocess
import os
import numpy as np

from fwk.wutils import parseargs

from PyTrilinos import Teuchos

import katoptron.utilities as u


def evaluate_prec(msh, comm, file_dir, muelu_xml):

    work_dir = os.getcwd()

    p = model.defaut_params
    p['MueLu xml use template'] = False

    N = 1
    num_random_variables = 20
    randomVariable = np.zeros((N, num_random_variables))

    pm = model.plate_model(ensemble_size, randomVariable,
                           msh, comm, file_dir, work_dir, p)
    pm.run(args.k)


def evaluate_all_prec(msh, comm, file_dir, muelu_xml_in, prec_params):

    N = prec_params.shape[0]

    for i in range(0, N):
        directory = str(i)
        if not os.path.exists(directory):
            os.makedirs(directory)
        os.chdir(directory)

        current_dir = os.getcwd()

        old_strings = ['$BLOCK_00_STRIDING_INFO', '$BLOCK_11_STRIDING_INFO',
                       '$BLOCK_00_GS_sweeps', '$BLOCK_11_GS_sweeps',
                       '$BLOCK_00_GS_damping', '$BLOCK_11_GS_damping',
                       '$SCHUR_omega', '$SimpleSmoother_damping']
        new_strings = ['3', '3', str(int(prec_params[i, 0])), str(int(prec_params[i, 1])), str(
            prec_params[i, 2]), str(prec_params[i, 3]), str(prec_params[i, 4]), str(prec_params[i, 5])]

        u.replace_strings(muelu_xml_in, current_dir +
                          '/SIMPLE_gs.xml', old_strings, new_strings)

        evaluate_prec(msh, comm, file_dir, current_dir+'/SIMPLE_gs.xml')

        os.chdir('..')


def main():
    try:
        import mpi4py.MPI as mpi
        comm = mpi.COMM_WORLD
        rank = comm.rank
        siz = comm.size
        name = mpi.Get_processor_name()
        status = mpi.Status()
        print("info: MPI found")
    except:
        comm = None
        rank = 0
        siz = 1
        name = "noname"
        print("info: MPI not found => MPI disabled")

    args = parseargs()

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    # Units in the meshes are in mm
    msh = u.fuse_meshes('plate_1.msh', 'plate_1_tmp', 'plate_2.msh',
                        'plate_2_tmp', 'plate_fused', file_dir, work_dir)

    muelu_xml_in = file_dir + '/../preconditioners/SIMPLE_gs.xml.in'

    N_GS_sweeps_00 = 1
    N_GS_sweeps_11 = 1
    N_GS_damping_00 = 6
    N_GS_damping_11 = 6
    N_SS_omega = 6
    N_SS_damping = 6

    GS_sweeps_00_min = 10
    GS_sweeps_00_max = 11

    GS_sweeps_11_min = 10
    GS_sweeps_11_max = 11

    GS_damping_00_min = 0.9
    GS_damping_00_max = 1.5

    GS_damping_11_min = 0.9
    GS_damping_11_max = 1.5

    SS_omega_min = 0.7
    SS_omega_max = 1.1

    SS_damping_min = 0.8
    SS_damping_max = 1.2

    sweeps_00 = np.arange(GS_sweeps_00_min, GS_sweeps_00_max, N_GS_sweeps_00)
    sweeps_11 = np.arange(GS_sweeps_11_min, GS_sweeps_11_max, N_GS_sweeps_11)
    damping_00_min = np.linspace(
        GS_damping_00_min, GS_damping_00_max, N_GS_damping_00)
    damping_11_min = np.linspace(
        GS_damping_11_min, GS_damping_11_max, N_GS_damping_11)
    SS_omega = np.linspace(SS_omega_min, SS_omega_max, N_SS_omega)
    SS_damping = np.linspace(SS_damping_min, SS_damping_max, N_SS_damping)

    N = N_GS_sweeps_00 * N_GS_sweeps_11 * N_GS_damping_00 * \
        N_GS_damping_11 * N_SS_omega * N_SS_damping

    prec_params = np.zeros((N, 6))

    i = 0

    for i_0 in range(0, N_GS_sweeps_00):
        for i_1 in range(0, N_GS_sweeps_11):
            for i_2 in range(0, N_GS_damping_00):
                for i_3 in range(0, N_GS_damping_11):
                    for i_4 in range(0, N_SS_omega):
                        for i_5 in range(0, N_SS_damping):
                            prec_params[i, :] = [sweeps_00[i_0], sweeps_11[i_1], damping_00_min[i_2],
                                                 damping_11_min[i_3], SS_omega[i_4], SS_damping[i_5]]
                            i = i + 1

    np.savetxt('prec_params.txt', prec_params)

    evaluate_all_prec(msh, comm, file_dir, muelu_xml_in, prec_params)


if __name__ == "__main__":
    main()
