#ifndef KATOPTRON_ELEMENTVECTORS_H
#define KATOPTRON_ELEMENTVECTORS_H

#include <Kokkos_Core.hpp>
#include "Kokkos_ViewFactory.hpp"
#include "wCacheHex8.h"
#include "wCacheQuad4.h"
#include "wCacheTetra4.h"
#include "wCacheTri3.h"
#include "tMatrix.h"
#include "Domain.h"
#include "Loads.h"

#include "ElementComputation.h"

#include "MaterialsList.h"
#include "Material.h"

#include "wLazy.h"
#include "tMatrix.h"

/**
 * @brief Class used to compute the matrix associated to the
 * integration of load on surface \f$\boldsymbol{N}\f$.
 */
template <typename scalar, int element_type, int element_size>
class ElementVectors : public ElementComputation<scalar, element_type>
{
public:
    Kokkos::View<scalar ***, Kokkos::LayoutRight, Kokkos::DefaultExecutionSpace> N;

    Teuchos::RCP<katoptron::Domain<scalar>> domain;

    size_t numPrimalDPN;

    ElementVectors(Teuchos::RCP<katoptron::Domain<scalar>> domain, size_t numPrimalDPN, size_t pool_size);

    void compute(int e, int i_thread);
    Kokkos::View<scalar **, Kokkos::LayoutRight, Kokkos::DefaultExecutionSpace> getN(size_t i_thread) const;
};

#endif //KATOPTRON_ELEMENTVECTORS_H
